import { ADMIN_FORMSERVICE } from '@admin-client/shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { AbstractFormService, createStateResource, isLoaded, StateResource } from '@alfa-client/tech-shared';
import { dispatchEventFromFixture, getMockComponent, mock, Mock, MockEvent } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker/.';
import { expect } from '@jest/globals';
import { Resource } from '@ngxp/rest';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { singleColdCompleted } from 'libs/tech-shared/test/marbles';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { AdminSaveButtonComponent } from './admin-save-button.component';

jest.mock('@alfa-client/tech-shared', () => {
  return {
    ...jest.requireActual('@alfa-client/tech-shared'),
    isLoaded: jest.fn(),
  };
});
const isLoadedMock: jest.Mock = isLoaded as jest.Mock;

describe('AdminSaveButtonComponent', () => {
  let component: AdminSaveButtonComponent;
  let fixture: ComponentFixture<AdminSaveButtonComponent>;

  let navigationService: Mock<NavigationService>;
  let formService: Mock<AbstractFormService<Resource>>;

  const saveButton: string = getDataTestIdOf('save');

  const stateResource: StateResource<Resource> = createStateResource(createDummyResource());

  beforeEach(async () => {
    formService = <any>{ submit: jest.fn().mockReturnValue(of(stateResource)) };
    navigationService = mock(NavigationService);

    await TestBed.configureTestingModule({
      imports: [AdminSaveButtonComponent],
      declarations: [MockComponent(ButtonWithSpinnerComponent)],
      providers: [
        {
          provide: ADMIN_FORMSERVICE,
          useValue: formService,
        },
        {
          provide: NavigationService,
          useValue: navigationService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(AdminSaveButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('on successful form submission', () => {
      it('should navigate to success link path', () => {
        const successLinkPath: string = faker.internet.url();
        component.successLinkPath = successLinkPath;
        isLoadedMock.mockReturnValue(true);

        component._navigateOnSuccessfulSubmission(createStateResource(createDummyResource()));

        expect(navigationService.navigate).toHaveBeenCalledWith(successLinkPath);
      });

      it('should NOT navigate', () => {
        isLoadedMock.mockReturnValue(false);

        component._navigateOnSuccessfulSubmission(createStateResource(createDummyResource()));

        expect(navigationService.navigate).not.toHaveBeenCalled();
      });
    });

    describe('on submit', () => {
      beforeEach(() => {
        component._navigateOnSuccessfulSubmission = jest.fn();
      });

      it('should call formService', () => {
        dispatchEventFromFixture(fixture, saveButton, MockEvent.CLICK);

        expect(formService.submit).toHaveBeenCalled();
      });

      it('should assign state resource', () => {
        dispatchEventFromFixture(fixture, saveButton, MockEvent.CLICK);

        expect(component.stateResource$).toBeObservable(singleColdCompleted(stateResource));
      });

      it('should navigate on successful submit', () => {
        dispatchEventFromFixture(fixture, saveButton, MockEvent.CLICK);
        component.stateResource$.subscribe();

        expect(component._navigateOnSuccessfulSubmission).toHaveBeenCalledWith(stateResource);
      });
    });
  });

  describe('button', () => {
    it('should call with stateResource', () => {
      component.stateResource$ = of(stateResource);

      fixture.detectChanges();

      const comp: ButtonWithSpinnerComponent = getMockComponent(fixture, ButtonWithSpinnerComponent);
      expect(comp.stateResource).toBe(stateResource);
    });
  });
});
