/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UserRepository } from '@admin/keycloak-shared';
import { createEmptyStateResource, createStateResource, StateResource } from '@alfa-client/tech-shared';
import { mock, Mock } from '@alfa-client/test-utils';
import { fakeAsync, TestBed, tick } from '@angular/core/testing';
import { cold } from 'jest-marbles';
import { singleCold } from 'libs/tech-shared/test/marbles';
import { Observable, of } from 'rxjs';
import { createUser } from '../../test/user';
import { User } from './user.model';
import { UserService } from './user.service';

import * as UserUtil from './user.util';

describe('UserService', () => {
  let service: UserService;
  let repository: Mock<UserRepository>;

  const user: User = createUser();

  beforeEach(() => {
    repository = { ...mock(UserRepository), getUsers: jest.fn().mockReturnValue(of([user])), getUserById: jest.fn() };

    TestBed.configureTestingModule({
      providers: [{ provide: UserRepository, useValue: repository }],
    });
    service = TestBed.inject(UserService);
  });

  describe('getItemsFromKeycloak', () => {
    it('should call getUsers from userRepository', () => {
      service._getItemsFromKeycloak();

      expect(repository.getUsers).toHaveBeenCalled();
    });

    it('should call sortUsersByLastName', fakeAsync(() => {
      const sortUsersByLastNameSpy: jest.SpyInstance<User[]> = jest.spyOn(UserUtil, 'sortUsersByLastName');

      service._getItemsFromKeycloak().subscribe();
      tick();

      expect(sortUsersByLastNameSpy).toHaveBeenCalled();
    }));
  });

  describe('createInKeycloak', () => {
    it('should call createInKeycloak from userRepository', () => {
      service._createInKeycloak(user);

      expect(repository.createInKeycloak).toHaveBeenCalledWith(user);
    });
  });

  describe('saveInKeycloak', () => {
    it('should call saveInKeycloak from userRepository', () => {
      service._saveInKeycloak(user);

      expect(repository.saveInKeycloak).toHaveBeenCalledWith(user);
    });
  });

  describe('getUserById', () => {
    const userStateResource: StateResource<User> = createStateResource(user);

    beforeEach(() => {
      repository.getUserById.mockReturnValue(singleCold(userStateResource, '-a'));
    });

    it('should call getUserById from userRepository', () => {
      service.getUserById(user.id);

      expect(repository.getUserById).toHaveBeenCalledWith(user.id);
    });

    it('should return loading state resource at first and response at second', () => {
      const user: Observable<StateResource<User>> = service.getUserById('userId');

      expect(user).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: userStateResource }));
    });
  });

  describe('deleteInKeycloak', () => {
    it('should call userRepository delete', () => {
      service._deleteInKeycloak(user.id);

      expect(repository.deleteUser).toHaveBeenCalledWith(user.id);
    });

    it('should return void', (done) => {
      repository.deleteUser.mockReturnValue(of(undefined));

      service._deleteInKeycloak(user.id).subscribe((result: void) => {
        expect(result).toBeUndefined();
        done();
      });
    });
  });
});
