/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  Bescheid,
  BescheidLinkRel,
  BescheidResource,
  BescheidSendBy,
  BescheidService,
  BescheidWizardStep,
  Wizard,
} from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { createErrorStateResource, createLoadingStateResource, formatForDatabase, StateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { UntypedFormBuilder } from '@angular/forms';
import { faker } from '@faker-js/faker';
import { expect } from '@jest/globals';
import { ResourceUri } from '@ngxp/rest';
import { EMPTY, Observable, of } from 'rxjs';
import { createBescheid, createBescheidResource, createWizard } from '../../../../bescheid-shared/src/test/bescheid';
import { createSuccessfullyDoneCommandStateResource } from '../../../../command-shared/test/command';
import { createProblemDetail } from '../../../../tech-shared/test/error';
import { singleCold, singleColdCompleted } from '../../../../tech-shared/test/marbles';
import { createVorgangWithEingangResource } from '../../../../vorgang-shared/test/vorgang';
import { BescheidFormService } from './bescheid.formservice';

registerLocaleData(localeDe);

describe('BescheidFormService', () => {
  let service: BescheidFormService;
  let bescheidService: Mock<BescheidService>;
  const now: Date = new Date();
  Date.now = jest.fn().mockReturnValue(now);
  const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource();

  beforeEach(() => {
    bescheidService = mock(BescheidService);
    service = new BescheidFormService(new UntypedFormBuilder(), useFromMock(bescheidService));
    service.setVorgangWithEingangResource(vorgangWithEingangResource);
  });

  describe('doSubmit', () => {
    const wizard: Wizard = createWizard();

    beforeEach(() => {
      service._submitBescheid = jest.fn().mockReturnValue(EMPTY);
      bescheidService.getWizard.mockReturnValue(of(wizard));
    });

    it('should get wizard', () => {
      service.submit().subscribe();

      expect(bescheidService.getWizard).toHaveBeenCalled();
    });

    it('should submit bescheid', () => {
      service.submit().subscribe();

      expect(service._submitBescheid).toHaveBeenCalledWith(wizard);
    });

    it('should return command', () => {
      const command: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
      service._submitBescheid = jest.fn().mockReturnValue(of(command));

      const submit$: Observable<StateResource<CommandResource>> = service.submit();

      expect(submit$).toBeObservable(singleColdCompleted(command));
    });
  });

  describe('_submitBescheid', () => {
    const command: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();

    beforeEach(() => {
      service._updateAndSendBescheid = jest.fn().mockReturnValue(EMPTY);
      service._createBescheid = jest.fn().mockReturnValue(EMPTY);
      service._updateBescheid = jest.fn().mockReturnValue(EMPTY);
      service.isPatch = jest.fn();
    });

    describe('create', () => {
      beforeEach(() => {
        service.isPatch = jest.fn().mockReturnValue(false);
      });

      it('should create', () => {
        service._submitBescheid(createWizard()).subscribe();

        expect(service._createBescheid).toHaveBeenCalled();
      });

      it('should return command', () => {
        service._createBescheid = jest.fn().mockReturnValue(of(command));

        const submit$: Observable<StateResource<CommandResource>> = service._submitBescheid(createWizard());

        expect(submit$).toBeObservable(singleColdCompleted(command));
      });
    });

    describe('update', () => {
      beforeEach(() => {
        service._canSendBescheid = jest.fn().mockReturnValue(false);
      });

      it('should update on change bescheid', () => {
        service.isPatch = jest.fn().mockReturnValue(true);

        service._submitBescheid(createWizard()).subscribe();

        expect(service._updateBescheid).toHaveBeenCalled();
      });

      it('should update on existing bescheid', () => {
        service.isPatch = jest.fn().mockReturnValue(false);
        service.patchValues(createBescheidResource());

        service._submitBescheid(createWizard()).subscribe();

        expect(service._updateBescheid).toHaveBeenCalled();
      });

      it('should return command', () => {
        service.isPatch = jest.fn().mockReturnValue(true);
        service._updateBescheid = jest.fn().mockReturnValue(of(command));

        const submit$: Observable<StateResource<CommandResource>> = service._submitBescheid(createWizard());

        expect(submit$).toBeObservable(singleColdCompleted(command));
      });
    });

    describe('update and send', () => {
      const wizard: Wizard = createWizard();

      beforeEach(() => {
        service.isPatch = jest.fn().mockReturnValue(true);
      });

      it('should update and send', () => {
        service._canSendBescheid = jest.fn().mockReturnValue(true);

        service._submitBescheid(wizard).subscribe();

        expect(service._updateAndSendBescheid).toHaveBeenCalledWith(wizard.sendBy);
      });

      it('should NOT update and send', () => {
        service._canSendBescheid = jest.fn().mockReturnValue(false);

        service._submitBescheid(wizard).subscribe();

        expect(service._updateAndSendBescheid).not.toHaveBeenCalled();
      });

      it('should return command', () => {
        service._updateAndSendBescheid = jest.fn().mockReturnValue(of(command));

        const submit$: Observable<StateResource<CommandResource>> = service._submitBescheid(wizard);

        expect(submit$).toBeObservable(singleColdCompleted(command));
      });
    });
  });

  describe('_canSendBescheid', () => {
    it('should return true', () => {
      const canSendBescheid: boolean = service._canSendBescheid({
        ...createWizard(),
        activeStep: BescheidWizardStep.BescheidVersenden,
        canBeSend: true,
      });

      expect(canSendBescheid).toBeTruthy();
    });

    it.each([BescheidWizardStep.AntragBescheiden, BescheidWizardStep.DokumenteHochladen])(
      'should return false on step %s',
      (step: BescheidWizardStep) => {
        const canSendBescheid: boolean = service._canSendBescheid({
          ...createWizard(),
          activeStep: step,
          canBeSend: true,
        });

        expect(canSendBescheid).toBeFalsy();
      },
    );

    it('should return false if bescheid sending locked', () => {
      const canSendBescheid: boolean = service._canSendBescheid({
        ...createWizard(),
        activeStep: BescheidWizardStep.BescheidVersenden,
        canBeSend: false,
      });

      expect(canSendBescheid).toBeFalsy();
    });
  });

  describe('_createBescheid', () => {
    beforeEach(() => {
      bescheidService.createBescheid.mockReturnValue(of(EMPTY));
    });

    it('should create bescheid', () => {
      const formValue: Bescheid = createBescheid();
      service.getBescheidFormValue = jest.fn().mockReturnValue(formValue);

      service._createBescheid();

      expect(bescheidService.createBescheid).toHaveBeenCalledWith(vorgangWithEingangResource, formValue);
    });

    it('should return create command', () => {
      const formValue: Bescheid = createBescheid();
      service.getBescheidFormValue = jest.fn().mockReturnValue(formValue);
      const createCommand: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
      bescheidService.createBescheid.mockReturnValue(of(createCommand));

      const command$: Observable<StateResource<CommandResource>> = service._createBescheid();

      expect(command$).toBeObservable(singleColdCompleted(createCommand));
    });
  });

  describe('_updateBescheid', () => {
    beforeEach(() => {
      bescheidService.updateBescheid.mockReturnValue(of(EMPTY));
    });

    it('should update bescheid', () => {
      const formValue: Bescheid = createBescheid();
      service.getBescheidFormValue = jest.fn().mockReturnValue(formValue);
      const bescheidResource: BescheidResource = createBescheidResource();
      service.patchValues(bescheidResource);

      service._updateBescheid();

      expect(bescheidService.updateBescheid).toHaveBeenCalledWith(bescheidResource, formValue);
    });

    it('should return create command', () => {
      const formValue: Bescheid = createBescheid();
      service.getBescheidFormValue = jest.fn().mockReturnValue(formValue);
      const bescheidResource: BescheidResource = createBescheidResource();
      service.patchValues(bescheidResource);
      const createCommand: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
      bescheidService.updateBescheid.mockReturnValue(of(createCommand));

      const command$: Observable<StateResource<CommandResource>> = service._updateBescheid();

      expect(command$).toBeObservable(singleColdCompleted(createCommand));
    });
  });

  describe('_updateAndSendBescheid', () => {
    beforeEach(() => {
      service._updateBescheid = jest.fn().mockReturnValue(EMPTY);
    });

    describe('send manually', () => {
      it('should update bescheid', () => {
        service._updateAndSendBescheid(BescheidSendBy.MANUAL).subscribe();

        expect(service._updateBescheid).toHaveBeenCalled();
      });

      it('should send bescheid manually', () => {
        service._updateBescheid = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

        service._updateAndSendBescheid(BescheidSendBy.MANUAL).subscribe();

        expect(bescheidService.sendBescheidManually).toHaveBeenCalled();
      });

      it('should not send on pending update', () => {
        service._updateBescheid = jest.fn().mockReturnValue(of(createLoadingStateResource()));

        service._updateAndSendBescheid(BescheidSendBy.MANUAL).subscribe();

        expect(bescheidService.sendBescheidManually).not.toHaveBeenCalled();
      });

      it('should not send on failed update', () => {
        service._updateBescheid = jest.fn().mockReturnValue(of(createErrorStateResource(createProblemDetail())));

        service._updateAndSendBescheid(BescheidSendBy.MANUAL).subscribe();

        expect(bescheidService.sendBescheidManually).not.toHaveBeenCalled();
      });

      it('should return update command on pending update', () => {
        const command: StateResource<CommandResource> = createLoadingStateResource();
        service._updateBescheid = jest.fn().mockReturnValue(of(command));

        const command$: Observable<StateResource<CommandResource>> = service._updateAndSendBescheid(BescheidSendBy.MANUAL);

        expect(command$).toBeObservable(singleColdCompleted(command));
      });

      it('should return update command on failed update', () => {
        const command: StateResource<CommandResource> = createErrorStateResource(createProblemDetail());
        service._updateBescheid = jest.fn().mockReturnValue(of(command));

        const command$: Observable<StateResource<CommandResource>> = service._updateAndSendBescheid(BescheidSendBy.MANUAL);

        expect(command$).toBeObservable(singleColdCompleted(command));
      });

      it('should return send bescheid command on successful update', () => {
        const command: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        service._updateBescheid = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));
        bescheidService.sendBescheidManually.mockReturnValue(of(command));

        const command$: Observable<StateResource<CommandResource>> = service._updateAndSendBescheid(BescheidSendBy.MANUAL);

        expect(command$).toBeObservable(singleColdCompleted(command));
      });

      it('should not send message', () => {
        service._updateBescheid = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

        service._updateAndSendBescheid(BescheidSendBy.MANUAL).subscribe();

        expect(bescheidService.sendBescheidMessage).not.toHaveBeenCalled();
      });
    });

    describe('send by nachricht', () => {
      it('should update bescheid', () => {
        service._updateAndSendBescheid(BescheidSendBy.NACHRICHT);

        expect(service._updateBescheid).toHaveBeenCalled();
      });

      it('should send bescheid message', () => {
        service._updateBescheid = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

        service._updateAndSendBescheid(BescheidSendBy.NACHRICHT).subscribe();

        expect(bescheidService.sendBescheidMessage).toHaveBeenCalled();
      });

      it('should not send on pending update', () => {
        service._updateBescheid = jest.fn().mockReturnValue(of(createLoadingStateResource()));

        service._updateAndSendBescheid(BescheidSendBy.NACHRICHT).subscribe();

        expect(bescheidService.sendBescheidMessage).not.toHaveBeenCalled();
      });

      it('should not send on failed update', () => {
        service._updateBescheid = jest.fn().mockReturnValue(of(createErrorStateResource(createProblemDetail())));

        service._updateAndSendBescheid(BescheidSendBy.NACHRICHT).subscribe();

        expect(bescheidService.sendBescheidMessage).not.toHaveBeenCalled();
      });

      it('should return update command on pending update', () => {
        const command: StateResource<CommandResource> = createLoadingStateResource();
        service._updateBescheid = jest.fn().mockReturnValue(of(command));

        const command$: Observable<StateResource<CommandResource>> = service._updateAndSendBescheid(BescheidSendBy.NACHRICHT);

        expect(command$).toBeObservable(singleColdCompleted(command));
      });

      it('should return update command on failed update', () => {
        const command: StateResource<CommandResource> = createErrorStateResource(createProblemDetail());
        service._updateBescheid = jest.fn().mockReturnValue(of(command));

        const command$: Observable<StateResource<CommandResource>> = service._updateAndSendBescheid(BescheidSendBy.NACHRICHT);

        expect(command$).toBeObservable(singleColdCompleted(command));
      });

      it('should return send bescheid command on successful update', () => {
        const command: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        service._updateBescheid = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));
        bescheidService.sendBescheidMessage.mockReturnValue(of(command));

        const command$: Observable<StateResource<CommandResource>> = service._updateAndSendBescheid(BescheidSendBy.NACHRICHT);

        expect(command$).toBeObservable(singleColdCompleted(command));
      });

      it('should not send manually', () => {
        service._updateBescheid = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

        service._updateAndSendBescheid(BescheidSendBy.NACHRICHT).subscribe();

        expect(bescheidService.sendBescheidManually).not.toHaveBeenCalled();
      });
    });
  });

  describe('patchValues', () => {
    const sendBy: BescheidSendBy = BescheidSendBy.NACHRICHT;
    let bescheidResource: BescheidResource;
    let patch: jest.Mock;

    beforeEach(() => {
      bescheidResource = createBescheidResource();
      patch = service.patch = jest.fn();
      service._evaluateSendBy = jest.fn().mockReturnValue(sendBy);
    });

    it('should patch', () => {
      const sendBy: BescheidSendBy = BescheidSendBy.NACHRICHT;
      service.patchValues(bescheidResource);

      expect(patch).toHaveBeenCalledWith({
        [BescheidFormService.FIELD_BESCHIEDEN_AM]: bescheidResource.beschiedenAm,
        [BescheidFormService.FIELD_BEWILLIGT]: String(bescheidResource.bewilligt),
        [BescheidFormService.FIELD_BESCHEID_DOCUMENT]: null,
        [BescheidFormService.FIELD_SEND_BY]: String(sendBy),
        [BescheidFormService.FIELD_NACHRICHT_SUBJECT]: bescheidResource.nachrichtSubject,
        [BescheidFormService.FIELD_NACHRICHT_TEXT]: bescheidResource.nachrichtText,
      });
    });

    it('should set send by', () => {
      const sendBy: BescheidSendBy = BescheidSendBy.NACHRICHT;
      service.patchValues(bescheidResource);

      expect(bescheidService.setSendBy).toHaveBeenCalledWith(sendBy);
    });

    it('should evaluate send by', () => {
      service.patchValues(bescheidResource);

      expect(service._evaluateSendBy).toHaveBeenCalledWith(bescheidResource);
    });

    it('should patch bescheid document uri', () => {
      const bescheidDocumentUri: ResourceUri = faker.internet.url();
      service.patchValues({
        ...bescheidResource,
        _links: {
          ...bescheidResource._links,
          [BescheidLinkRel.BESCHEID_DOCUMENT]: { href: bescheidDocumentUri },
        },
      });

      expect(patch).toHaveBeenCalledWith({
        [BescheidFormService.FIELD_BESCHIEDEN_AM]: bescheidResource.beschiedenAm,
        [BescheidFormService.FIELD_BEWILLIGT]: String(bescheidResource.bewilligt),
        [BescheidFormService.FIELD_SEND_BY]: sendBy,
        [BescheidFormService.FIELD_BESCHEID_DOCUMENT]: bescheidDocumentUri,
        [BescheidFormService.FIELD_NACHRICHT_SUBJECT]: bescheidResource.nachrichtSubject,
        [BescheidFormService.FIELD_NACHRICHT_TEXT]: bescheidResource.nachrichtText,
      });
    });

    it('should patch attachments', () => {
      service.patchValues(bescheidResource);

      expect(service.getFormValue().attachments).toEqual(bescheidResource.attachments);
    });
  });

  describe('_evaluateSendBy', () => {
    it('should return MANUAL on MANUAL', () => {
      const sendBy: BescheidSendBy = service._evaluateSendBy({
        ...createBescheidResource([BescheidLinkRel.BESCHEIDEN]),
        sendBy: BescheidSendBy.MANUAL,
      });

      expect(sendBy).toEqual(BescheidSendBy.MANUAL);
    });

    it('should return MANUAL on NACHRICHT and missing link', () => {
      const sendBy: BescheidSendBy = service._evaluateSendBy({
        ...createBescheidResource([BescheidLinkRel.BESCHEIDEN]),
        sendBy: BescheidSendBy.NACHRICHT,
      });

      expect(sendBy).toEqual(BescheidSendBy.MANUAL);
    });

    it('should return NACHRICHT', () => {
      const sendBy: BescheidSendBy = service._evaluateSendBy({
        ...createBescheidResource([BescheidLinkRel.BESCHEIDEN_UND_SENDEN]),
        sendBy: BescheidSendBy.NACHRICHT,
      });

      expect(sendBy).toEqual(BescheidSendBy.NACHRICHT);
    });

    it('should return MANUAL', () => {
      const sendBy: BescheidSendBy = service._evaluateSendBy({
        ...createBescheidResource([BescheidLinkRel.BESCHEIDEN_UND_SENDEN]),
        sendBy: BescheidSendBy.MANUAL,
      });

      expect(sendBy).toEqual(BescheidSendBy.MANUAL);
    });
  });

  describe('getBescheidFormValueChanges', () => {
    const bescheid: Bescheid = createBescheid();

    beforeEach(() => {
      service.getBescheidFormValue = jest.fn().mockReturnValue(bescheid);
    });

    it('should emit bescheid', () => {
      const bescheidFormValueChanges$: Observable<Bescheid> = service.getBescheidFormValueChanges();

      expect(bescheidFormValueChanges$).toBeObservable(singleCold(bescheid));
    });
  });

  describe('getBescheidFormValue', () => {
    beforeEach(() => {
      service.getFormValue = jest.fn().mockReturnValue({ beschiedenAm: now, bewilligt: 'true' });
    });

    it('should getFormValue', () => {
      service.getBescheidFormValue();

      expect(service.getFormValue).toHaveBeenCalled();
    });

    it('should return bescheid', () => {
      const bescheid: Bescheid = service.getBescheidFormValue();

      expect(bescheid).toEqual({
        bewilligt: true,
        beschiedenAm: formatForDatabase(now),
      } as Bescheid);
    });
  });

  describe('clearBescheidDocumentFile', () => {
    it('should update bescheid document file', () => {
      service.updateBescheidDocumentFile = jest.fn();

      service.clearBescheidDocumentFile();

      expect(service.updateBescheidDocumentFile).toHaveBeenCalledWith(null);
    });
  });

  describe('updateBescheidDocumentFile', () => {
    it('should patch form control value to null', () => {
      service.updateBescheidDocumentFile();

      expect(service.getBescheidFormValue().bescheidDocument).toBeNull();
    });

    it('should patch form control value', () => {
      const bescheidResource: BescheidResource = createBescheidResource();

      service.updateBescheidDocumentFile(bescheidResource.bescheidDocument);

      expect(service.getBescheidFormValue().bescheidDocument).toEqual(bescheidResource.bescheidDocument);
    });
  });
});
