import {
  checkBoxGroupsEmptyValidator,
  fieldEmptyValidator,
  fieldInvalidValidator,
  fieldLengthValidator,
  InvalidParam,
} from '@alfa-client/tech-shared';
import { AbstractControl, FormControl, UntypedFormGroup } from '@angular/forms';
import { expect } from '@jest/globals';
import { ValidationMessageCode } from './tech.validation.messages';

describe('Tech Validators', () => {
  describe('field empty validator', () => {
    it('should return null', () => {
      const control: AbstractControl = new FormControl('test');

      expect(fieldEmptyValidator('')(control)).toBeNull();
    });

    it('should return invalid param', () => {
      const control: AbstractControl = new FormControl(null);

      expect(fieldEmptyValidator('')(control)).toEqual({
        [ValidationMessageCode.FIELD_EMPTY]: {
          name: '',
          value: null,
          reason: ValidationMessageCode.FIELD_EMPTY,
          constraintParameters: [],
        } as InvalidParam,
      });
    });
  });

  describe('field invalid validator', () => {
    it('should return null', () => {
      const control: AbstractControl = new FormControl('test');

      expect(fieldInvalidValidator('', /^test$/)(control)).toBeNull();
    });

    it('should return invalid param', () => {
      const control: AbstractControl = new FormControl('test2');

      expect(fieldInvalidValidator('', /^test$/)(control)).toEqual({
        [ValidationMessageCode.FIELD_INVALID]: {
          name: '',
          value: null,
          reason: ValidationMessageCode.FIELD_INVALID,
          constraintParameters: [],
        } as InvalidParam,
      });
    });
  });

  describe('field length validator', () => {
    it('should return null', () => {
      const control: AbstractControl = new FormControl('test');

      expect(fieldLengthValidator('', 1, 5)(control)).toBeNull();
    });

    it('should return invalid param with min length', () => {
      const control: AbstractControl = new FormControl('t');

      expect(fieldLengthValidator('', 2, 5)(control)).toEqual({
        [ValidationMessageCode.FIELD_SIZE]: {
          name: '',
          value: null,
          reason: ValidationMessageCode.FIELD_SIZE,
          constraintParameters: [
            { name: 'min', value: '2' },
            { name: 'max', value: '5' },
          ],
        } as InvalidParam,
      });
    });

    it('should return invalid param with max length', () => {
      const control: AbstractControl = new FormControl('test test test');

      expect(fieldLengthValidator('', 2, 5)(control)).toEqual({
        [ValidationMessageCode.FIELD_SIZE]: {
          name: '',
          value: null,
          reason: ValidationMessageCode.FIELD_SIZE,
          constraintParameters: [
            { name: 'min', value: '2' },
            { name: 'max', value: '5' },
          ],
        } as InvalidParam,
      });
    });
  });

  describe('check box group empty validator', () => {
    it('should return null', () => {
      const control: UntypedFormGroup = new UntypedFormGroup({
        group1: new UntypedFormGroup({
          check1: new FormControl(true),
          check2: new FormControl(false),
        }),
        group2: new UntypedFormGroup({
          check1: new FormControl(false),
          check2: new FormControl(false),
        }),
      });

      expect(checkBoxGroupsEmptyValidator('', ['group1', 'group2'])(control)).toBeNull();
    });

    it('should return invalid param', () => {
      const control: UntypedFormGroup = new UntypedFormGroup({
        group1: new UntypedFormGroup({
          check1: new FormControl(false),
          check2: new FormControl(false),
        }),
        group2: new UntypedFormGroup({
          check1: new FormControl(false),
          check2: new FormControl(false),
        }),
      });

      expect(checkBoxGroupsEmptyValidator('', ['group1', 'group2'])(control)).toEqual({
        [ValidationMessageCode.FIELD_EMPTY]: {
          name: '',
          value: null,
          reason: ValidationMessageCode.FIELD_EMPTY,
          constraintParameters: [],
        } as InvalidParam,
      });
    });

    it('should return invalid param and ignore group', () => {
      const control: UntypedFormGroup = new UntypedFormGroup({
        group1: new UntypedFormGroup({
          check1: new FormControl(false),
          check2: new FormControl(false),
        }),
        group2: new UntypedFormGroup({
          check1: new FormControl(true),
          check2: new FormControl(false),
        }),
      });

      expect(checkBoxGroupsEmptyValidator('', ['group1'])(control)).toEqual({
        [ValidationMessageCode.FIELD_EMPTY]: {
          name: '',
          value: null,
          reason: ValidationMessageCode.FIELD_EMPTY,
          constraintParameters: [],
        } as InvalidParam,
      });
    });
  });
});
